# Copyright (C) 2015 Emmanuel Garette <gnunux@gnunux.info>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#OpenOffice.org Registry Format (OOR)
#informations: http://util.openoffice.org/common/configuration/oor-document-format.html

import xml.etree.ElementTree as ET
from xml.etree.ElementTree import Element, ElementTree
from xml.etree.ElementPath import xpath_tokenizer
from os.path import dirname, isdir, isfile
from os import environ
from gaspacho_agent.util import makedirs, chown
from gaspacho_agent.util import logger, trace


gasp_version = "0.1"

#fork from etree/ElementPath.py
def prepare_predicate(next, token):
    # FIXME: replace with real parser!!! refs:
    # http://effbot.org/zone/simple-iterator-parser.htm
    # http://javascript.crockford.com/tdop/tdop.html
    signature = []
    predicate = []
    while 1:
        token = next()
        if token[0] == "]":
            break
        if token[0] and token[0][:1] in "'\"":
            token = "'", token[0][1:-1]
        signature.append(token[0] or "-")
        predicate.append(token[1])
    signature = "".join(signature)
    # use signature to determine predicate type
    if signature == "@-":
        # [@attribute] predicate
        raise ValueError('attribut without value not supported')
    if signature == "@-='":
        # [@attribute='value']
        key = predicate[1]
        value = predicate[-1]
        return key, value
    if signature == "-" and not re.match("\d+$", predicate[0]):
        # [tag]
        raise ValueError('tag without value not supported')
    if signature == "-='" and not re.match("\d+$", predicate[0]):
        # [tag='value']
        raise ValueError('tag without value not supported')
    if signature == "-" or signature == "-()" or signature == "-()-":
        # [index] or [last()] or [last()-index]
        raise ValueError('tag without value not supported')
    raise SyntaxError("invalid predicate")

class Apply:
    @trace
    def __init__(self, conflevel, section_separator='/'):
        self.section_separator = section_separator
        self.conflevel = conflevel

    @trace
    def is_compatible(self):
        return True

    @trace
    def initialize(self):
        self.path = None
        self.xml = None
        if self.conflevel == 'context':
            self.uid = None
            self.gid = None
        else:
            self.uid = int(environ['UID'])
            self.gid = int(environ['GID'])

    @trace
    def open(self, path):
        if self.path != None:
            raise Exception('use close() before it')
        self.path = path
        #try:
        if isfile(path):
            self.xml = ET.parse(self.path)
        else:
            self.xml = None

    @trace
    def set(self, key, value, info):
        #info should be like that: Inet/Settings

        infos = info.split(self.section_separator)
        if self.xml is not None:
            node = self.xml.getroot()
            if node.tag != infos[0]:
                raise Exception('root is not {0}'.format(infos[0]))
        else:
            node = Element(infos[0])
            self.xml = ElementTree(node)
        for index, section in enumerate(infos):
            if index == 0:
                continue
            cur_node = node.find(section)

            if cur_node is None:
                #if no node
                #if xpath
                next = iter(xpath_tokenizer(section)).next
                token = next()
                attr = None
                while 1:
                    print token
                    try:
                        if token[0] == '':
                            section_name = token[1]
                        elif token[0] == '[':
                            attr = prepare_predicate(next, token)
                        else:
                            raise ValueError('{0} not supported in xpath'.format(token[0]))
                    except StopIteration:
                        raise SyntaxError("invalid path")
                    try:
                        token = next()
                        if token[0] == "/":
                            token = next()
                    except StopIteration:
                        break
                new_node = Element(section_name)
                if attr is not None:
                    new_node.attrib[attr[0]] = attr[1]
                node.append(new_node)
                node = new_node
            else:
                node = cur_node
        if key == '':
            node.text = value
        else:
            node.attrib[key] = value

    @trace
    def suppr(self, key, info):
        if self.path == None:
            raise Exception('use open() before it')
        raise NotImplemented('unsupported')

    @trace
    def close(self):
        if self.path == None:
            raise Exception('already close')
        dirn = dirname(self.path)
        if not isdir(dirn):
            makedirs(dirn, uid=self.uid, gid=self.gid)
        #write and change right
        self.xml.write(self.path, encoding='UTF-8')
        if self.uid is not None:
            chown(self.path, self.uid, self.gid)
        self.path = None

    @trace
    def finalize_without_target(self):
        pass

    @trace
    def finalize(self):
        pass

# vim: ts=4 sw=4 expandtab
